;***********************************************************************
PRO NewT,T

  common Tdepend, K_H, K_He, K_Fe, A_H, A_He, A_Fe   ; common block
;-----------------------------------------------------------------------

; Some constants   
  h     = 6.626e-34                                  ; Planck
  m     = 9.109e-31                                  ; electron mass
  k_B   = 1.38054e-23                                ; Boltzmann
  e     = 1.602e-19                                  ; electron charge
  Xi_H  = 13.6                                       ; Hydrogen ion.pot.
  Xi_He = 22.4                                       ; Helium ion.pot.
  Xi_Fe = 7.9                                        ; Iron ion.pot.
  A_H  = 1.0                                         ; Hydrogen normalized
  A_He = 0.1                                         ; Helium relative H
  A_Fe = 1e-4                                        ; Iron relative H
 ;A_Fe = 0

; Calculate temperature dependent factors for the ionization balance
  void = check_math()                                ; clear math errors
  a    = e/(k_B*T)                                   ; one ev = t_ev kelvin
; c    = 0.5*((h^2/(2.*pi*k_B*m*T))^1.5              ; avoid under/overflow
; c    = 0.5*((h^2/(2.*!pi*k_B*m*T)))^1.5            ; ### CORRECTED SYNTAX ERROR: MISSING PARENTHESIS AND ! IN FRONT OF pi
  c    = 0.5*((h/k_B)*(h/(2.*!pi*m*T)))^1.5          ; ### CORRECTED FLOATING ILLEGAL OPERAND ERROR: REWROTE EXPRESSION 

  if (check_math(mask=32) ne 0) then stop            ; uncomment to check for underflow

; Note: we are simplifying things here by using the same degeneracy factor for all three elements.  
; For hydrogen, which is most important, the factor is one, from the degeneracy of the free electron 
; and the bound electron.

  gfact = 1.

; K_H  = gfact*c*exp(Xi_H *a)                        ; hydrogen
; K_He = gfact*c*exp(Xi_He*a)                        ; helium
; K_Fe = gfact*c*exp(Xi_Fe*a)                        ; iron
  K_H  = 1/(gfact*c)*exp(-Xi_H *a)                   ; ### CORRECTED OVERFLOW ERROR: REWROTE TO 1/K EXPRESSION
  K_He = 1/(gfact*c)*exp(-Xi_He*a)                   ; ### CORRECTED OVERFLOW ERROR: REWROTE TO 1/K EXPRESSION
  K_Fe = 1/(gfact*c)*exp(-Xi_Fe*a)                   ; ### CORRECTED OVERFLOW ERROR: REWROTE TO 1/K EXPRESSION

  if (check_math(mask=64) ne 0) then stop            ; uncomment to check for overflow
END

;***********************************************************************
FUNCTION Ne_over_N, N_e

; Calculate the particle density ratio N/N_e, given the electron number density N_e. 
; Temperature dependent K_ factors from common block.


  common Tdepend, K_H, K_He, K_Fe, A_H, A_He, A_Fe
;-----------------------------------------------------------------------

; y = (1. + (A_H + A_He + A_Fe)/                     ; cf. Equation (37)
;   (A_H /(1.+N_e*K_H ) + $                          ; in the lectur notes
;    A_He/(1.+N_e*K_He) + $
;    A_Fe/(1.+N_e*K_Fe)))
; y = (1. + (A_H + A_He + A_Fe)/ $                   ; ### CORRECTED SYNTAX ERROR: MISSING $
;   (A_H /(1.+N_e*K_H ) + $
;    A_He/(1.+N_e*K_He) + $
;    A_Fe/(1.+N_e*K_Fe)))
  y = (1. + (A_H + A_He + A_Fe)/ $                   ; ### CORRECTED OVERFLOW ERROR: REWROTE TO 1/K EXPRESSION 
    (A_H*K_H  /(K_H+N_e ) + $
     A_He*K_He/(K_He+N_e) + $
     A_Fe*K_Fe/(K_Fe+N_e)))
  return, 1./y
END

;***********************************************************************
;FUNCTION residue, N_e
FUNCTION residue, ln_N_e                             ; ### WORK WITH LOGARITMS  

  common density, N                                  ; N from main prog
;-----------------------------------------------------------------------

  N_e = exp(ln_N_e)                                  ; ### WORK WITH LOGARITMS: CALCULATING BACK TO N_e

  plots, N_e, N_e/N, psym=1, symsize=0.5, col=0.6*!d.table_size               ; ### PLOT THE PROCESS
  plots, N_e, Ne_over_N(N_e), psym=1, symsize=0.5, col=0.75*!d.table_size     ; ###   -||-
  wait,0.001                                                                  ; ### WAIT 0.001 s BEFORE CONTINUING 

  f = Ne_over_N(N_e)/(N_e/N)                         ; =1 when N(N_e)=N
; f = (N_e/N)/Ne_over_N(N_e)                         ; ### OPPOSITE FRACTION IN LECTURE NOTES??...         ####################################?
; return, 1.-f                                       ; =0 when N(N_e)=N
  return, alog(f)                                    ; ### WORK WITH LOGARITMS ON THE y-AXIS  
END

;***********************************************************************
;  Main IDL program.
;  Calculates the ionization balance for hydrogen, helium and iron.

  common Tdepend, K_H, K_He, K_Fe, A_H, A_He, A_Fe
  common density, N                                  ; pass to residue

  on_error, 0                                        ; stop at trouble location

  k_B     = 1.38054e-23                              ; Boltzmann
  min_T   =  2000.                                   ; min temperature
  max_T   = 15000.                                   ; max temperature
  delta_T =   500.                                   ; T values
  min_lg_P   = -2.0                                  ; min log pressure
  max_lg_P   =  8.0                                  ; max log pressure
  delta_lg_P =  0.5                                  ; P values

  nT = (max_T    - min_T   )/delta_T    + 1          ; T dimension
  nT = long(nT)                                      ; make sure integer
  nP = (max_lg_P - min_lg_P)/delta_lg_P + 1          ; T dimension
  nP = long(nP)                                      ; make sure integer
  tol = 1.e-4                                        ; tolerance

  TT  = fltarr(nT)                                   ; allocate T array
  lgP = fltarr(nP)                                   ; allocate P array
  f_e = fltarr(nT,nP)                                ; array for result

;-----------------------------------------------------------------------

  f_e_max = 0.501                                    ; if all ionized

  Ntot = 0
  for iT=0,nT-1 do begin                             ; temperature loop <-----.
                                                     ;                        |
    T  = min_T + delta_T*iT                          ; temperature            |
    TT(iT) = T                                       ; array for plot         |
    NewT,T                                           ; T dept factors         |
                                                     ;                        |
    plot, [1,1e30],[1e-10,1], $                      ; -----------------------+--> ### PLOT THE PROCESS
       /xlog, /ylog, /nodata                         ;                        |
                                                     ;                        |
    for iP=0,nP-1 do begin                           ; pressure loop <-----.  |
                                                     ;                     |  |
      lgP(iP) = min_lg_P + iP*delta_lg_P             ; for plot            |  |
;     lgP(iP) = min_lg_P + (iP-1)*delta_lg_P         ; --------------------+--+--> ### DIFFERENT IN LECTURE NOTES??    ########################?
      P = 10.^lgP(iP)                                ; pressure            |  |
      N = P/(k_B*T)                                  ; number dens         |  |
                                                     ;                     |  |
;     Ne_max = f_e_max*N                             ; upper bound --------+--+--> ### OLD UPPER BOUND
;     Ne_min = 1e-10                                 ; lower bound --------+--+--> ### OLD LOWER BOUND
                                                     ;                     |  |
      a = N * 1/K_H * (1. + A_He)                    ;                     |  |
      b = (2. + A_He)                                ;                     |  |
      c = -1.                                        ;                     |  |
      Ne_min = N*c/(-0.5 * (b + sqrt(b^2 - 4.*a*c))) ; ### BETTER WITH THIS LOWER BOUND (3.7 AVERAGE ITERATIONS)
                                                     ;
      Ne_max = 1.0001 * N * sqrt( Ne_over_N(N) )     ; ### EVEN BETTER WITH THIS UPPER BOUND (3.2 AVERAGE ITERATIONS)
                                                     ;                     |  |
;     N_e = zbrent1 ('residue', $                    ;                     |  |
;            Ne_min, Ne_max, tol, Nit)               ; call root finder    |  |
      N_e = exp( zbrent1 ('residue', $               ; --------------------+--+--> ### WORK WITH LOGARITMS
             alog(Ne_min), alog(Ne_max), tol, Nit))  ; call root finder    |  |
                                                     ;                     |  |
      f_e(it,iP) = N_e/N                             ; store ratio         |  |
      Ntot = Ntot + Nit                              ; count iterations    |  |
                                                     ;                     |  |
     end                                             ; end pressure loop <-'  |
                                                     ;                        |
    print,'T =',T,'      Nit =',Nit                  ;                        |
  end                                                ; end temperature loop <-'

  print,'Average no of iterations =',float(Ntot)/(nP*nT)

  surface, f_e, TT, lgP, $                           ; surface plot
    xstyle=1, ystyle=1, ztype=1, $                   ; exact axes, log scale
    xtitle='T', ytitle='log P', $                    ; axes titles
    ztitle='N!de!n/N', charsize=1.4                  ; note subscript
  print, f_e(4,2)
END



; NÅET TIL: If you have time, think about ways of improving the initial values. Der er tre spørgsmål tilbage..